//
// Tizen Native SDK
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Flora License, Version 1.1 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://floralicense.org/license/
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an AS IS BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <new>
#include <FGraphics.h>
#include <FMedia.h>
#include "AppResourceId.h"
#include "NfcMemoTag.h"
#include "MainForm.h"

using namespace Tizen::Base;
using namespace Tizen::Base::Collection;
using namespace Tizen::Base::Utility;
using namespace Tizen::Ui;
using namespace Tizen::Ui::Controls;
using namespace Tizen::Ui::Scenes;
using namespace Tizen::App;
using namespace Tizen::Net::Nfc;
using namespace Tizen::Graphics;

const static int NFC_MEMO_TAG_LABEL_COLOR_R = 30;
const static int NFC_MEMO_TAG_LABEL_COLOR_G = 250;
const static int NFC_MEMO_TAG_LABEL_COLOR_B = 30;

MainForm::MainForm(void)
	: __pTagStatus(null)
	, __pReadForm(null)
	, __pWriteForm(null)
	, __pNfcManager(null)
	, __pNdefTagConnection(null)
	, __pTimerPopup(null)
	, __pOkPopup(null)
	, __pTimerPopupLabel(null)
	, __pOkPopupLabel(null)
	, __popupTimer()
{

}

MainForm::~MainForm(void)
{
	delete __pTimerPopup;
	delete __pOkPopup;
}

bool
MainForm::Initialize(void)
{
	// Construct an XML form
	Construct(IDF_MAINFORM);

	return true;
}

result
MainForm::OnInitializing(void)
{
	result r = E_SUCCESS;
	SetName(L"MainForm");

	//Construct popup
	InitializePopups();

	__pNfcManager = new (std::nothrow) NfcManager();

	//Construct NFCManager and INfcManagerEventListener which provide event NFC Activate and Deactivate.
	r = __pNfcManager->Construct(*this);

	//Add Tag Discovery event listener, which provides event for tag connections,
	//you can use filter for what type of tag you want events.
	r = __pNfcManager->AddTagDiscoveryEventListener(*this, NFC_TAG_TYPE_ALL);

	//Add Message Discovery event listener, which provide event when tag sends any message.
	//you can filter for what type of message you want events.
	r = __pNfcManager->AddNdefMessageDiscoveryEventListener(*this, NdefRecordType(NDEF_TNF_ALL));

	__pTagStatus = static_cast< Label* >(GetControl(IDC_TAG_STATUS));
	__pTagStatus->SetTextConfig(40, LABEL_TEXT_STYLE_BOLD);
	__pTagStatus->SetTextHorizontalAlignment(ALIGNMENT_CENTER);
	__pTagStatus->SetTextColor(Tizen::Graphics::Color(205, 30, 30));
	__pTagStatus->SetTextVerticalAlignment(ALIGNMENT_MIDDLE);
	__pTagStatus->SetText(L"No Tag Connected");

	Footer* pFooter = GetFooter();
	if (pFooter != null)
	{
		pFooter->SetStyle(FOOTER_STYLE_BUTTON_TEXT);
		r = __leftFooterItem.Construct(ID_ACTIVATE);

		//Find whether NFC is active or not.
		if (__pNfcManager->IsActivated() == true)
		{
			r = __leftFooterItem.SetText("Deactivate");
		}
		else
			r = __leftFooterItem.SetText("Activate");
		r = pFooter->AddItem(__leftFooterItem);

		r = __middleFooterItem.Construct(ID_READ_MEMO);
		r = __middleFooterItem.SetText("Read Memo");
		r = pFooter->AddItem(__middleFooterItem);
		pFooter->SetItemEnabled(1, false);

		r = __rightFooterItem.Construct(ID_WRITE_MEMO);
		r = __rightFooterItem.SetText("Write Memo");
		r = pFooter->AddItem(__rightFooterItem);
		SetFormBackEventListener(this);
		pFooter->AddActionEventListener(*this);
	}
	//check if Tag is connected
	if (__pNfcManager->IsTagConnected() == true)
	{
		TagConnection* pConnection = __pNfcManager->GetCurrentTagConnectionN();
		if (pConnection)
		{
			//check if it's Ndef connection or not
			if (pConnection->IsNdefConnection() == true)
			{
				__pNdefTagConnection = (NdefTagConnection*) pConnection;

				//Add Connection Listener to Tag connection, which provides indication for read complete and write complete
				__pNdefTagConnection->SetNdefTagConnectionListener(this);

				__pTagStatus->SetTextColor(Tizen::Graphics::Color(NFC_MEMO_TAG_LABEL_COLOR_R, NFC_MEMO_TAG_LABEL_COLOR_G,
						NFC_MEMO_TAG_LABEL_COLOR_B));
				__pTagStatus->SetText(L"Tag Connected");
				GetFooter()->SetItemEnabled(1, true);
				RequestRedraw();
			}
			else
			{
				ShowTimerPopup(L"Tag Detected", L"New Tag Connected\n But it's not NDEF tag.");
				delete pConnection;
			}
		}
		else
		{
			AppLog("Failed to get TagConnection.");
		}
	}

	return r;
}

result
MainForm::OnTerminating(void)
{
	result r = E_SUCCESS;
	if (__pNdefTagConnection != null)
	{
		delete __pNdefTagConnection;
		__pNdefTagConnection = null;
	}

	if (__pNfcManager != null)
	{
		//Remove Tag Discovery Listener.
		__pNfcManager->RemoveTagDiscoveryEventListener(*this, NFC_TAG_TYPE_ALL);
		//Remove Message Discovery Listener.
		__pNfcManager->RemoveNdefMessageDiscoveryEventListener(*this, NdefRecordType(NDEF_TNF_ALL));
		delete __pNfcManager;
		__pNfcManager = null;
	}

	return r;
}
void
MainForm::LoadReadForm(NdefMessage* pMsg)
{
	SceneManager* pSceneManager = SceneManager::GetInstance();
	AppAssert(pSceneManager);
	ArrayList* pParams = null;
	pParams = new (std::nothrow) ArrayList();
	pParams->Add(*pMsg);

	if(pSceneManager->GetCurrentSceneId() != "ReadScene")
	{
		pSceneManager->GoForward(ForwardSceneTransition(L"ReadScene"), pParams);
	}
	else
	{
		Form* pReadForm = pSceneManager->GetCurrentScene()->GetForm();
		pReadForm->SendUserEvent(ReadForm::REQUEST_UPDATE, pParams);
	}
}

void
MainForm::LoadWriteForm()
{
	SceneManager* pSceneManager = SceneManager::GetInstance();
	AppAssert(pSceneManager);

	pSceneManager->GoForward(ForwardSceneTransition(L"WriteScene"));
}

void
MainForm::OnActionPerformed(const Tizen::Ui::Control& source, int actionId)
{
	result r = E_SUCCESS;

	switch (actionId)
	{
	case ID_ACTIVATE:


		if (__pNfcManager->IsActivated() == true)
		{
			AppLog("Deactivate Button is clicked!!");
			r = __pNfcManager->Deactivate();
			if (r != E_SUCCESS)
			{
				AppLog("Failed to deactivate NFC %s", GetErrorMessage(r));
			}
		}
		else
		{
			AppLog("Activate Button is clicked!!");
			result r = __pNfcManager->Activate();
			if (r != E_SUCCESS)
			{
				AppLog("Failed to activate NFC %s", GetErrorMessage(r));
			}
		}
		break;

	case ID_WRITE_MEMO:
		AppLog("Write Memo Button Clicked");
		LoadWriteForm();
		break;

	case ID_READ_MEMO:
		AppLog("Read Memo Button Clicked");
		if (__pNdefTagConnection != null)
		{
			result r = E_SUCCESS;
			r = __pNdefTagConnection->Read();
			if (r != E_SUCCESS)
			{
				AppLog("Failed to read the data %s", GetErrorMessage(r));
			}
		}
		else
		{
			AppLog("Connection was lost");
		}

		break;

	case ID_OK:
		HideOkPopup();
		break;

	default:
		break;
	}
}

void
MainForm::OnFormBackRequested(Form& source)
{
	UiApp* pApp = UiApp::GetInstance();
	AppAssert(pApp);
	pApp->Terminate();
}

void
MainForm::OnNfcActivated(result r)
{
	//indication from framework when NFC is activated with result of event.

	if (IsFailed(r))
	{
		AppLog("NFC Activation failed");
	}
	else
	{
		AppLog("NFC Activated");
		__leftFooterItem.SetText("Deactivate");
		Footer* pFooter = GetFooter();
		if (pFooter != null)
		{
			pFooter->RemoveItemAt(0);
			pFooter->InsertItemAt(0, __leftFooterItem);
			RequestRedraw();
		}

		ShowTimerPopup(L"NFC Activate", L"NFC Successfully Activated.");
	}
}

void
MainForm::OnNfcDeactivated(result r)
{
	//indication from framework when NFC is deactivated with result of event.
	if (IsFailed(r))
	{
		AppLog("NFC Deactivation failed");
	}
	else
	{
		AppLog("NFC Deactivated");
		__leftFooterItem.SetText("Activate");
		Footer* pFooter = GetFooter();
		if (pFooter != null)
		{
			pFooter->RemoveItemAt(0);
			pFooter->InsertItemAt(0, __leftFooterItem);
			RequestRedraw();
		}

		ShowTimerPopup(L"NFC Deactivate", L"NFC Successfully Deactivated.");
	}
}

void
MainForm::OnNfcTagDetectedN(Tizen::Net::Nfc::TagConnection* pConnection)
{
	//indication from frame work when Tag is connected and parameter as connection to that Tag

	AppLog("Tag Detected");

	//check if it's Ndef connection or not
	if (pConnection->IsNdefConnection() == true)
	{
		__pNdefTagConnection = (NdefTagConnection*) pConnection;

		//Add Connection Listener to Tag connection, which provides indication for read complete and write complete
		__pNdefTagConnection->SetNdefTagConnectionListener(this);

		__pTagStatus->SetTextColor(Tizen::Graphics::Color(NFC_MEMO_TAG_LABEL_COLOR_R, NFC_MEMO_TAG_LABEL_COLOR_G,
				NFC_MEMO_TAG_LABEL_COLOR_B));
		__pTagStatus->SetText(L"Tag Connected");
		GetFooter()->SetItemEnabled(1, true);
		RequestRedraw();
	}
	else
	{
		ShowTimerPopup(L"Tag Detected", L"New Tag Connected\n But it's not NDEF tag.");
		delete pConnection;
	}
}

void
MainForm::OnNfcTagLost(void)
{
	//Indication from framework for NFC tag disconnected.
	AppLog("Tag Lost");

	if (__pNdefTagConnection != null)
	{
		delete __pNdefTagConnection;
		__pNdefTagConnection = null;
	}
	__pTagStatus->SetTextColor(Tizen::Graphics::Color(250, 30, 30));
	__pTagStatus->SetText(L"No Tag Connected");
	GetFooter()->SetItemEnabled(1, false);
	RequestRedraw();
}

void
MainForm::OnNdefMessageDetectedN(Tizen::Net::Nfc::NdefMessage* pMessage)
{
	//Indication from frame work for message received from Tag.
	AppLog("Some message detected");
	LoadReadForm(pMessage);
}

void
MainForm::OnNdefReadCompletedN(Tizen::Net::Nfc::NdefMessage* pMessage, result r)
{
	//Indication of Read operation completed which parameter pmessage as data read and result of operation as r.
	AppLog("OnNdefReadCompletedN %s", GetErrorMessage(r));
	if (r == E_SUCCESS)
	{
		LoadReadForm(pMessage);
	}
	else
	{
		ShowOkPopup(L"Read Operation", L"Read Data failed");
	}
}

void
MainForm::OnNdefWriteCompleted(result r)
{
	//Indication of Write operation completed, result r denotes the result of operation.
	AppLog("OnNdefWriteCompleted %s", GetErrorMessage(r));

	String msgdata;

	switch (r)
	{
	case E_SUCCESS:
		msgdata.Append(L"Successful.");
		break;

	case E_ILLEGAL_ACCESS:
		msgdata.Append(L"Failed. Writing is denied by the tag.");
		break;

	case E_STORAGE_FULL:
		msgdata.Append(L"Failed. Memo is too long.");
		break;

	case E_NOT_RESPONDING:
		msgdata.Append(L"Failed. The tag is not responding.");
		break;

	case E_SYSTEM:
	default:
		msgdata.Append(L"Failed. Internal error.");
		break;
	}

	ShowOkPopup(L"Write Operation", msgdata);
}

void
MainForm::OnSceneActivatedN(const Tizen::Ui::Scenes::SceneId& previousSceneId,
							   const Tizen::Ui::Scenes::SceneId& currentSceneId, Tizen::Base::Collection::IList* pArgs)
{
	if(pArgs)
	{
		WriteData((NdefMessage*) pArgs->GetAt(0));
		delete pArgs;
	}
}

void
MainForm::OnSceneDeactivated(const Tizen::Ui::Scenes::SceneId& currentSceneId,
								const Tizen::Ui::Scenes::SceneId& nextSceneId)
{

}

void
MainForm::OnTimerExpired(Tizen::Base::Runtime::Timer& timer)
{
	if (__popupTimer.Equals(timer))
	{
		__pTimerPopup->SetShowState(false);
		Draw(true);
	}
}

void
MainForm::WriteData(NdefMessage* pMsg)
{
	if (__pNdefTagConnection == null)
	{
		ShowTimerPopup(L"NFC Tag Connection", L"No Tag Connected\n Write Failed.");
		pMsg->RemoveAllRecords(true);
		delete pMsg;
		return;
	}


	//Write Message over NFC Tag, indication of successful operation would be received at OnWriteCompleted.
	result r = __pNdefTagConnection->Write(*pMsg);
	if (r != E_SUCCESS)
	{
		AppLog("Error in writing %s", GetErrorMessage(r));
	}
	else
	{
		AppLog("Write successfully");
	}
	pMsg->RemoveAllRecords(true);
	delete pMsg;
}

void
MainForm::InitializePopups(void)
{
	static const int POPUP_BUTTON_WIDTH = 100;
	static const int POPUP_BUTTON_HEIGHT = 60;
	static const int POPUP_MARGIN = 30;

	__pTimerPopup = new Popup();
	__pTimerPopup->Construct(true, Dimension(GetClientAreaBounds().width - POPUP_MARGIN*2, GetClientAreaBounds().height / 5));

	Rectangle rect;
	rect = __pTimerPopup->GetClientAreaBounds();
	__pTimerPopupLabel = new (std::nothrow) Label();
	__pTimerPopupLabel->Construct(Rectangle(0,0,rect.width, rect.height), L"");
	__pTimerPopupLabel->SetTextHorizontalAlignment(ALIGNMENT_LEFT);
	__pTimerPopupLabel->SetTextConfig(25, LABEL_TEXT_STYLE_NORMAL);
	__pTimerPopup->AddControl(__pTimerPopupLabel);
	__popupTimer.Construct(*this);

	__pOkPopup = new Popup();
	__pOkPopup->Construct(true, Dimension(GetClientAreaBounds().width, GetClientAreaBounds().height / 3));

	rect = __pOkPopup->GetClientAreaBounds();
	__pOkPopupLabel = new (std::nothrow) Label();
	__pOkPopupLabel->Construct(Rectangle(0,0,rect.width, rect.height - 70), L"");
	__pOkPopupLabel->SetTextHorizontalAlignment(ALIGNMENT_LEFT);
	__pOkPopupLabel->SetTextConfig(25, LABEL_TEXT_STYLE_NORMAL);
	__pOkPopup->AddControl(__pOkPopupLabel);

	Button* pButtonOk = new Button();
	pButtonOk->Construct(Rectangle(((__pOkPopup->GetClientAreaBounds().width / 2) - POPUP_BUTTON_HEIGHT),( __pOkPopup->GetClientAreaBounds().height / 2), POPUP_BUTTON_WIDTH, POPUP_BUTTON_HEIGHT), L"Ok");
	pButtonOk->SetActionId(ID_OK);
	pButtonOk->AddActionEventListener(*this);
	__pOkPopup->AddControl(*pButtonOk);
}

void
MainForm::ShowTimerPopup(const Tizen::Base::String& title, const Tizen::Base::String& message)
{
	__pTimerPopupLabel->SetText(message);
	__pTimerPopup->SetTitleText(title);
	__pTimerPopup->SetShowState(true);
	__pTimerPopup->Show();

	__popupTimer.Start(POPUP_TIMEOUT);
}

void
MainForm::ShowOkPopup(const Tizen::Base::String& title, const Tizen::Base::String& message)
{
	__pOkPopupLabel->SetText(message);
	__pOkPopup->SetTitleText(title);
	__pOkPopup->SetShowState(true);
	__pOkPopup->Show();
}

void
MainForm::HideOkPopup(void)
{
	AppLog("Ok Button Clicked");
	__pOkPopup->SetShowState(false);
	Draw(true);
}
