//
// Tizen Native SDK
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Flora License, Version 1.1 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://floralicense.org/license/
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an AS IS BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <new>
#include "AppResourceId.h"
#include "MainForm.h"
#include "ReadForm.h"
#include <FGraphics.h>
#include <FMedia.h>
#include <FApp.h>

using namespace Tizen::Base;
using namespace Tizen::Base::Utility;
using namespace Tizen::Ui;
using namespace Tizen::Ui::Controls;
using namespace Tizen::Ui::Scenes;
using namespace Tizen::Graphics;
using namespace Tizen::Media;
using namespace Tizen::App;
using namespace Tizen::Net::Nfc;

const static byte NFC_MEMO_TAG_ENCODING_TYPE_MASK = 0x80;
const static byte NFC_MEMO_TAG_CODE_LENGTH_MASK = 0x3F;
const static wchar_t* TABLE_RTD_URI[] = {
L"",
L"http://www.",
L"https://www.",
L"http://",
L"https://",
L"tel:",
L"mailto:",
L"ftp://anonymous:anonymous@",
L"ftp://ftp.",
L"ftps://",
L"sftp://",
L"smb://",
L"nfs://",
L"ftp://",
L"dav://",
L"news:",
L"telnet://",
L"imap:",
L"rtsp://",
L"urn:",
L"pop:",
L"sip:",
L"sips:",
L"tftp:",
L"btspp://",
L"btl2cap://",
L"btgoep://",
L"tcpobex://",
L"irdaobex://",
L"file://",
L"urn:epc:id:",
L"urn:epc:tag:",
L"urn:epc:pat:",
L"urn:epc:raw:",
L"urn:epc:",
L"urn:nfc:"
};

ReadForm::ReadForm(void)
	: __pLeftItemBitmap(null)
	, __pRightItemBitmap(null)
	, __pRecordTypeField(null)
	, __pRecordNameField(null)
	, __pPayloadArea(null)
	, __pMsg(null)
	, __totalRecordCount(-1)
	, __currentRecord(-1)
{

}

ReadForm::~ReadForm(void)
{
	if (__pMsg != null)
	{
		__pMsg->RemoveAllRecords(true);
		delete __pMsg;
	}

	if (__pLeftItemBitmap != null)
	{
		delete __pLeftItemBitmap;
		__pLeftItemBitmap = null;
	}

	if (__pRightItemBitmap != null)
	{
		delete __pRightItemBitmap;
		__pRightItemBitmap = null;
	}
}

bool
ReadForm::Initialize()
{
	Form::Construct(IDF_READFORM);

	return true;
}

result
ReadForm::SetupData(NdefMessage* pMsg)
{
	result r = E_SUCCESS;
	if (__pMsg != null)
	{
		__pMsg->RemoveAllRecords(true);
		delete __pMsg;
		__pMsg = null;
	}
	__pMsg = new (std::nothrow) NdefMessage(*pMsg);

	__totalRecordCount = __pMsg->GetRecordsCount();
	__currentRecord = -1;
	Header* pHeader = GetHeader();
	if (pHeader != null)
	{
		pHeader->RemoveAllButtons();

		if (__totalRecordCount > 1)
		{
			Image* pImage = new (std::nothrow) Image();

			r = pImage->Construct();
			if (r != E_SUCCESS)
			{
				AppLog("Error in Image construct");
				return r;
			}
			__pLeftItemBitmap = pImage->DecodeN(App::GetInstance()->GetAppRootPath() + L"/res/Left_arrow.png", BITMAP_PIXEL_FORMAT_ARGB8888);
			if (__pLeftItemBitmap == null)
			{
				AppLog("Unable to Load Left button image, not creating form now...");
				return E_FAILURE;
			}
			__pRightItemBitmap = pImage->DecodeN(App::GetInstance()->GetAppRootPath() + L"/res/Right_arrow.png", BITMAP_PIXEL_FORMAT_ARGB8888);
			if (__pRightItemBitmap == null)
			{
				AppLog("Unable to Load Right button image, not creating form now...");
				return E_FAILURE;
			}

			ButtonItem buttonItemLeft;
			buttonItemLeft.Construct(BUTTON_ITEM_STYLE_ICON, ID_HEADER_PREV);
			buttonItemLeft.SetIcon(BUTTON_ITEM_STATUS_NORMAL, __pLeftItemBitmap);

			ButtonItem buttonItemRight;
			buttonItemRight.Construct(BUTTON_ITEM_STYLE_ICON, ID_HEADER_NEXT);
			buttonItemRight.SetIcon(BUTTON_ITEM_STATUS_NORMAL, __pRightItemBitmap);

			pHeader->SetStyle(HEADER_STYLE_TITLE);
			pHeader->SetTitleText(L"Read NDEF message");
			pHeader->SetButton(BUTTON_POSITION_LEFT, buttonItemLeft);
			pHeader->SetButton(BUTTON_POSITION_RIGHT, buttonItemRight);
			delete pImage;
			pHeader->AddActionEventListener(*this);
		}
	}
	return r;
}

void
ReadForm::UpdateRecord(int increment)
{

	if ((__currentRecord + increment) >= 0 && (__currentRecord + increment) < __totalRecordCount)
	{

		__currentRecord = __currentRecord + increment;
		NdefRecord* pRecord = __pMsg->GetRecordAt(__currentRecord);
		TryReturnVoid(pRecord != null, "Record doesn't exist");

		__pRecordNameField->SetText(pRecord->GetRecordType().GetName());



		switch (pRecord->GetRecordType().GetNameFormat())
		{
		case NDEF_TNF_EMPTY:
			__pRecordTypeField->SetText(L"Ndef Empty");
			break;

		case NDEF_TNF_WELL_KNOWN:
			__pRecordTypeField->SetText(L"Ndef Well Known");
			break;

		case NDEF_TNF_MIME_MEDIA:
			__pRecordTypeField->SetText(L"Ndef Mime Media");
			break;

		case NDEF_TNF_ABSOLUTE_URI:
			__pRecordTypeField->SetText(L"Ndef Absolute URI");
			break;

		case NDEF_TNF_EXTERNAL:
			__pRecordTypeField->SetText(L"Ndef External");
			break;

		case NDEF_TNF_UNKNOWN:
			__pRecordTypeField->SetText(L"Ndef Unknown");
			break;

		default:
			__pRecordTypeField->SetText(L"Invalid");
			break;

		}

		if (pRecord->GetPayload() != null)
		{
			String* pTextPayload;

			pTextPayload = GetTextPayloadN(pRecord->GetPayload(), pRecord->GetRecordType().GetName());
			if (pTextPayload != null)
			{
				__pPayloadArea->SetText(*pTextPayload);
				delete pTextPayload;
			}
		}
		else
		{
			__pPayloadArea->SetText(L"");
		}

		String description;
		description.Append(__currentRecord + 1);
		description.Append('/');
		description.Append(__totalRecordCount);
		Header* pHeader = GetHeader();
		if (pHeader != null)
			pHeader->SetDescriptionText(description);
		RequestRedraw();
	}
}

String*
ReadForm::GetTextPayloadN(const ByteBuffer* pData, String ndefRecordName)
{
	//parsing the record as per the RTD format.
	String textPayload;
	ByteBuffer copiedData;
	copiedData.Construct(*pData);

	//check if record is of Text Type
	if (ndefRecordName == NFC_NDEF_RTD_TEXT)
	{

		byte statusByte;
		int languageCodeLen;

		copiedData.GetByte(statusByte);
		String encodingType = (statusByte & NFC_MEMO_TAG_ENCODING_TYPE_MASK) ? L"UTF-16" : L"UTF-8";
		languageCodeLen = statusByte & NFC_MEMO_TAG_CODE_LENGTH_MASK;
		byte languageCode[languageCodeLen + 1];
		result r = copiedData.GetArray(languageCode, 0, (int) languageCodeLen);
		if (r != E_SUCCESS)
		{
			AppLog("Invalid encoding, return original string");
			StringUtil::Utf8ToString((char*) copiedData.GetPointer(), textPayload);
			return new (std::nothrow) String(textPayload);
		}
		languageCode[languageCodeLen] = '\0';
		AppLog("Language Code %s", languageCode);
		if (encodingType == L"UTF-16")
		{
			//UTF-16 encoding
			McharBuffer* pUtf16Data = copiedData.AsMcharBufferN();
			StringUtil::MbToString(*pUtf16Data, textPayload);
			delete pUtf16Data;
		}
		else
		{

			//UTF-8 encoding
			int payloadLen = copiedData.GetRemaining();
			AppLog("Payload Length is %d", payloadLen);
			char byteData[payloadLen + 1];
			r = copiedData.GetArray((byte*) byteData, 0, payloadLen);
			byteData[payloadLen] = '\0';
			AppLog("%s", byteData);
			StringUtil::Utf8ToString(byteData, textPayload);
		}
		__pRecordNameField->SetText(L"Text");
		return new (std::nothrow) String(textPayload);
	}
	//Check if the encoding is URI
	else if (ndefRecordName == NFC_NDEF_RTD_URI)
	{
		String uriPayload;
		int uriTypeLength = 1;
		//First byte specifies URI type.
		byte statusByte;
		copiedData.GetByte(statusByte);
		int uriType = (int) statusByte;
		if (uriType > 35)
		{
			uriType = 0;
			uriTypeLength = 0;
		}

		int lenPayload = copiedData.GetRemaining();
		AppLog("Payload Length is %d", lenPayload);
		byte byteData[lenPayload + 1];
		copiedData.GetArray(byteData, 0, (int) lenPayload);
		byteData[lenPayload] = '\0';
		AppLog("%s", byteData);

		StringUtil::Utf8ToString((char*) (byteData), uriPayload);
		uriPayload.Insert(TABLE_RTD_URI[uriType], 0);
		__pRecordNameField->SetText(L"URI");
		return new (std::nothrow) String(uriPayload);
	}
	//in case encoding is other then Text and URI then display data directly.
	else
	{
		AppLog("Unknown RTD, data is %s", pData->GetPointer());
		int payloadLen = copiedData.GetRemaining();
		AppLog("Payload Length is %d", payloadLen);
		char byteData[payloadLen + 1];
		copiedData.GetArray((byte*) byteData, 0, payloadLen);
		byteData[payloadLen] = '\0';
		AppLog("%s", byteData);
		StringUtil::Utf8ToString(byteData, textPayload);
		return new (std::nothrow) String(textPayload);
	}
}

result
ReadForm::OnInitializing(void)
{
	result r = E_SUCCESS;
	SetName(L"ReadForm");
	SetFormBackEventListener(this);

	__pRecordTypeField = static_cast< EditField* >(GetControl(IDC_RECORDTYPE));
	TryReturn(__pRecordTypeField != null, E_FAILURE, "Can't get __pRecordTypeField Handle");

	__pRecordTypeField->SetKeypadEnabled(false);
	__pRecordTypeField->SetViewModeEnabled(true);

	__pRecordNameField = static_cast< EditField* >(GetControl(IDC_RECORDNAME));
	TryReturn(__pRecordNameField != null, E_FAILURE, "Can't get __pRecordNameField Handle");

	__pRecordNameField->SetKeypadEnabled(false);
	__pRecordNameField->SetViewModeEnabled(true);

	__pPayloadArea = static_cast< EditArea* >(GetControl(IDC_PAYLOAD));
	TryReturn(__pPayloadArea != null, E_FAILURE, "Can't get __pPayloadArea Handle");

	__pPayloadArea->SetKeypadEnabled(false);
	__pPayloadArea->SetViewModeEnabled(true);


	return r;
}

result
ReadForm::OnTerminating(void)
{
	result r = E_SUCCESS;

	return r;
}

void
ReadForm::OnSceneActivatedN(const Tizen::Ui::Scenes::SceneId& previousSceneId,
							   const Tizen::Ui::Scenes::SceneId& currentSceneId, Tizen::Base::Collection::IList* pArgs)
{
	SetupData((NdefMessage*) pArgs->GetAt(0));
	UpdateRecord();
	RequestRedraw();

	pArgs->RemoveAll(true);
	delete pArgs;
}

void
ReadForm::OnSceneDeactivated(const Tizen::Ui::Scenes::SceneId& currentSceneId,
								const Tizen::Ui::Scenes::SceneId& nextSceneId)
{

}

void
ReadForm::OnActionPerformed(const Control& source, int actionId)
{
	switch (actionId)
	{

	case ID_HEADER_PREV:
		AppLog("Previous Button Pressed");
		UpdateRecord(-1);
		break;

	case ID_HEADER_NEXT:
		AppLog("Next Button Pressed");
		UpdateRecord();
		break;

	default:
		break;
	}
}

void
ReadForm::OnUserEventReceivedN(RequestId requestId, Tizen::Base::Collection::IList* pArgs)
{
	switch (requestId)
	{
	case REQUEST_UPDATE:

		SetupData((NdefMessage*) pArgs->GetAt(0));
		UpdateRecord();
		RequestRedraw();

		pArgs->RemoveAll(true);
		delete pArgs;
		break;

	default:
		break;
	}
}

void
ReadForm::OnFormBackRequested(Tizen::Ui::Controls::Form& source)
{
	SceneManager* pSceneManager = SceneManager::GetInstance();
	AppAssert(pSceneManager);

	pSceneManager->GoForward(ForwardSceneTransition(L"MainScene"));
}
